'''
Author: LetMeFly
Date: 2022-01-25 18:07:26
LastEditors: LetMeFly
LastEditTime: 2024-06-03 23:32:43
'''
# __all__ = ['AutoGetAndSubmit', 'FromPidToChar', 'GetASourceCodeAndChange', 'GetContestProblemList', 'Login', 'SubmitOneCode']
from .Login import login
from .AutoGetAndSubmit import main as finish1contest
from .SubmitOneCode import submit, submitById
from .EditingAProbem import create1problem, edit1problem
from .DownloadAllCodes import saveAllCodes



'''
Author: LetMeFly
Date: 2022-02-21 15:39:38
LastEditors: LetMeFly
LastEditTime: 2022-02-21 17:00:52
'''

global info
info = {
    "base_url": "https://buctcoder.com/"
}

def set_info(key: str, value: str) -> None:
    info[key] = value

def get_info(key: str) -> str:
    return info.get(key, None)



'''
Author: LetMeFly
Date: 2022-01-24 15:55:51
LastEditors: LetMeFly
LastEditTime: 2024-06-03 21:32:00
'''
import requests


def submit(cid: str, pid: str, cookies: requests.cookies.RequestsCookieJar,
           code: str) -> requests.Response:
    from . import Config
    base_url = Config.get_info("base_url")
    url = f"{base_url}submit.php"
    data = {
        'cid': cid,
        'pid': pid,
        'language': '1',  # C++
        'source': code
    }
    response = requests.post(url, data=data, cookies=cookies)
    return response


def submitById(cookies: requests.cookies.RequestsCookieJar, id: str, code: str,
               language: str) -> None:
    from . import Config
    base_url = Config.get_info("base_url")
    url = f"{base_url}submit.php"
    languageTable = {
        "C": "0",
        "C++": "1",
        "Java": "3",
        "Python": "6",
        "PHP": "7",
        "JavaScript": "16",
        "Go": "17",
        "SQL": "18",
    }
    if language in languageTable:
        language = languageTable[language]
    data = {
        'id': id,
        'language': language,
        'source': code,
    }
    response = requests.post(url, data=data, cookies=cookies)
    return response



'''
Author: LetMeFly
Date: 2022-03-09 17:19:51
LastEditors: LetMeFly
LastEditTime: 2022-03-10 18:14:06
'''
from bs4 import BeautifulSoup
from typing import Tuple
import requests


def get_postkey_and_csrf(cookies: requests.cookies.RequestsCookieJar) -> Tuple[str, str]:
    """
    通过Admin的Cookie获取新建问题需要的postkey和csrf
    """
    from . import Config
    base_url = Config.get_info("base_url")
    add_problem_url = base_url + "admin/problem_add_page.php"
    response_postkey = requests.get(url=add_problem_url, cookies=cookies)
    soup_postkey = BeautifulSoup(response_postkey.text, "lxml")
    postkey = soup_postkey.find("input", attrs={"name": "postkey"}).get("value")
    
    csrf_url = base_url + "csrf.php"
    response_csrf = requests.get(url=csrf_url, cookies=cookies)
    soup_csrf = BeautifulSoup(response_csrf.text, "lxml")
    csrf = soup_csrf.find("input").get("value")
    
    return (postkey, csrf)


def edit1problem(cookies: requests.cookies.RequestsCookieJar, problem_id: str, title: str, description: str, input: str, output: str, sample_input = "", sample_output = "", hint = "", time_limit = 1, memory_limit = 128, spj = 0, source = "", contest_id = "") -> None:
    """
    修改一道已经存在的题目

    Parameters:
        problem_id - 要修改的题目的id
        其他 - 参考create1problem函数
    """
    postkey, csrf = get_postkey_and_csrf(cookies)

    from . import Config
    base_url = Config.get_info("base_url")
    url = base_url + 'admin/problem_edit.php'
    data = {
        "problem_id": problem_id,
        "title": title,
        "time_limit": time_limit,
        "memory_limit": memory_limit,
        "description": description,
        "input": input,
        "output": output,
        "sample_input": sample_input,
        "sample_output": sample_output,
        "test_input": "",
        "test_output": "",
        "hint": hint,
        "spj": spj,
        "source": source,
        "contest_id": contest_id,
        "postkey": postkey,
        "submit": "保存",
        "csrf": csrf
    }
    response = requests.post(url=url, cookies=cookies, data=data)


def create1problem(cookies: requests.cookies.RequestsCookieJar, title: str, description: str, input: str, output: str, sample_input = "", sample_output = "", hint = "", time_limit = 1, memory_limit = 128, spj = 0, source = "", contest_id = "") -> str:
    """
    通过Admin的Cookie创建一道新的题目
    
    Parameters:
        cookies - 管理号的cookies，可通过from BUCTOJ import login来获得
        title - 题目
        time_limit - 时间限制，默认时间为1s
        memory_limit - 内存限制，默认为128M
        description - 题目描述
        input - 输入描述
        output - 输出描述
        sample_input - 样例输入，默认为空
        sample_output - 样例输出，默认为空
        hint - 提示，默认为空
        spj - 是否为special judge， 默认为0
        source - 来源/分类，默认为空
        contest_id - 属于哪场竞赛/作业，默认为空
    
    Returns:
        problem_id - 新创建的问题的problem_id
    """
    postkey, csrf = get_postkey_and_csrf(cookies)

    from . import Config
    base_url = Config.get_info("base_url")
    url = base_url + 'admin/problem_add.php'
    data = {
        "problem_id": "New Problem",
        "title": title,
        "time_limit": time_limit,
        "memory_limit": memory_limit,
        "description": description,
        "input": input,
        "output": output,
        "sample_input": sample_input,
        "sample_output": sample_output,
        "test_input": "",
        "test_output": "",
        "hint": hint,
        "spj": spj,
        "source": source,
        "contest_id": contest_id,
        "postkey": postkey,
        "submit": "保存",
        "csrf": csrf
    }
    response = requests.post(url=url, cookies=cookies, data=data)
    soup = BeautifulSoup(response.text, "lxml")
    href = soup.find("a").get("href")
    problem_id = href.split("javascript:phpfm(")[1].split(");")[0]

    edit1problem(cookies=cookies, problem_id=problem_id, title=title, description=description, input=input, output=output, sample_input=sample_input, sample_output=sample_output, hint=hint, time_limit=time_limit, memory_limit=memory_limit, spj=spj, source=source, contest_id=contest_id)
    return problem_id





'''
Author: LetMeFly
Date: 2024-06-03 23:24:40
LastEditors: LetMeFly
LastEditTime: 2024-06-03 23:34:21
'''
import requests
from bs4 import BeautifulSoup
try:
    import lxml
    features = 'lxml'
except:
    features = None
from os.path import exists
from os import mkdir


def get1page(url) -> list:
    """
    获取一页的所有提交，返回“所有提交的列表”
    列表中每个元素都是html的tr
    """
    response = requests.get(url)
    soup = BeautifulSoup(response.text, 'lxml')
    table = soup.find('table', attrs={"id": "result-tab"})
    trs = table.find('tbody').find_all('tr')
    return trs


def getAllpages(baseurl):
    results = []

    thisPage = get1page(baseurl)
    while True:
        results += thisPage
        if not thisPage or len(thisPage) == 1:
            break
        prevtop = thisPage[0].find('td').get_text()
        top = thisPage[-1].find('td').get_text()
        print(top, prevtop)
        newUrl = baseurl + f'&top={top}&pretop={prevtop}'
        thisPage = get1page(newUrl)
        if thisPage:
            thisPage = thisPage[1:]
    return results


def get1codeBySubmissionId(submissionId, cookies):
    from . import Config
    base_url = Config.get_info("base_url")
    response = requests.get(f'{base_url}showsource.php?id={submissionId}', cookies=cookies)
    soup = BeautifulSoup(response.text, 'lxml')
    code = soup.find('pre')
    return code.get_text()


def saveAllCodes(baseurl, cookies):
    results = getAllpages(baseurl)
    print(f'Total num: {len(results)}')
    if not exists('results'):
        mkdir('results')
    for thisSubmission in results:
        submissionId = thisSubmission.find('td').get_text()
        userid = thisSubmission.find_all('td')[1].get_text()
        print(f'Getting {userid}\'s submssion {submissionId}')
        code = get1codeBySubmissionId(submissionId, cookies=cookies)
        with open(f'results/{userid}-{submissionId}.java', 'w', encoding='utf-8') as f:
            f.write(code)

